/************************************************************************\
 *
 * VaultWiki
 * Version 3.0.17
 *
 * Copyright 2008-2012 Cracked Egg Studios. All Rights Reserved.
 * This file may not be redistributed in whole or significant part.
 * 
 * VAULTWIKI IS NOT FREE SOFTWARE
 * http://www.vaultwiki.org/
 * http://www.vaultwiki.org/pages/License-Agreement
 *
\************************************************************************/

/**
* Initialize AJAX post editing
*
* @param	mixed	ID of element (or actual element) containing postbits
*/
function vB_AJAX_QuickSection_Init(postobj)
{
	if (AJAX_Compatible)
	{
		if (typeof postobj == 'string')
		{
			vB_QuickSection.qe_obj = postobj;
			postobj = fetch_object(postobj);
		}

		var anchors = fetch_tags(postobj, 'a');
		vB_QuickSection.postanchors = anchors;

		for (var i = 0; i < anchors.length; i++)
		{
			if (anchors[i].name && anchors[i].name.indexOf('vB::QuickSection::') != -1)
			{
				anchors[i].onclick = vB_AJAX_QuickSection_Events.prototype.editbutton_click;
			}
		}
	}
}

// #############################################################################
// vB_AJAX_QuickSection
// #############################################################################

/**
* Class to allow quick editing of posts within postbit via AJAX
*/
function vB_AJAX_QuickSection()
{
	this.ajax_load_url = null;
	this.postid = null;
	this.sectid = null;
	this.objid = null;
	this.messageobj = null;
	this.container = null;
	this.originalhtml = null;
	this.editstate = false;
	this.editorcounter = 0;
	this.ajax_req = null;
	this.show_advanced = true;
	this.postanchors = null;
	this.qe_obj = '';
	this.is_cke = (typeof vB_Text_Editor != 'undefined' && typeof vB_Text_Editor.prototype.autosave == 'function');
}

// =============================================================================
// vB_AJAX_QuickSection methods

/**
* Check if the AJAX system is ready for us to proceed
*
* @return	boolean
*/
vB_AJAX_QuickSection.prototype.ready = function()
{
	if (this.editstate || YAHOO.util.Connect.isCallInProgress(this.ajax_req))
	{
		return false;
	}
	else
	{
		return true;
	}
};

/**
* Prepare to edit a single post
*
* @param	string	Name attribute of clicked link - takes the form of 'vB::QuickSection::$postid:$sectionid'
*
* @return	boolean
*/
vB_AJAX_QuickSection.prototype.edit = function(anchor_name)
{
	if (typeof vb_disable_ajax != 'undefined' && vb_disable_ajax > 0)
	{
		// couldn't initialize, return true to allow click to go through
		return true;
	}

	this.qe_obj = '';
	var q;

	for (var i in section_holders)
	{
		var holder = YAHOO.util.Dom.get(section_holders[i]);
		if (!holder)
		{
			continue;
		}

		var holden = holder.getElementsByTagName("a");
		if (!holden)
		{
			continue;
		}

		for (q = 0; q < holden.length; q++)
		{
			if (holden[q].name == anchor_name)
			{
				this.qe_obj = holder.id;
				break;
			}
		}

		if (this.qe_obj != '')
		{
			break;
		}
	}
	
	if (this.qe_obj == '')
	{
		return true;
	}

	if (
		this.qe_obj == "forum_header" ||
		typeof vB_WikiTabs != "object" ||
		typeof vB_WikiTabs[tab_container] != "object"
	)
	{
		// ajax url not set
		this.splitter = ajax_load_url[this.qe_obj].indexOf("?") != -1 ? "?" : "&";
		this.ajax_load_url = ajax_load_url[this.qe_obj].split(this.splitter);
		this.ajax_load_url[1] = this.ajax_load_url[1].replace(/\{(\d+)(:\w+)?\}/gi, '%$1$s');
	}
	else
	{
		this.splitter = vB_WikiTabs[tab_container].splitter;
		this.ajax_load_url = vB_WikiTabs[tab_container].ajax_load_url;
	}

	for (var i = 0; i < this.postanchors.length; i++)
	{
		if (this.postanchors[i].name && this.postanchors[i].name.indexOf('vB::QuickSection::') != -1)
		{
			this.postanchors[i].onclick = '';
		}
	}

	var tmppostid = anchor_name.substr(anchor_name.lastIndexOf('::') + 2);
	var tmpsectid = tmppostid;

	tmpsectid = tmpsectid.substr(tmppostid.lastIndexOf(':') + 1);
	tmppostid = tmppostid.substring(0, tmppostid.lastIndexOf(':'));

	if (YAHOO.util.Connect.isCallInProgress(this.ajax_req))
	{
		// something is waiting to complete
		return false;
	}
	else if (!this.ready())
	{
		if (this.postid == tmppostid)
		{
			this.full_edit();
			return false;
		}
		this.abort();
	}

	this.editorcounter++;
	this.editorid = 'vB_Editor_QE_' + this.editorcounter;

	this.postid = tmppostid;
	this.sectid = tmpsectid;

	this.objid = 'collapseobj_' + this.postid + '_' + this.sectid;
	this.messageobj = fetch_object(this.objid);
	this.originalhtml = this.messageobj.innerHTML;

	this.unchanged = null;
	this.unchanged_reason = null;

	this.fetch_editor();
	this.editstate = true;

	return false;
};

/**
* Send an AJAX request to fetch the editor HTML
*/
vB_AJAX_QuickSection.prototype.fetch_editor = function()
{
	document.body.style.cursor = 'wait';
	var poststarttime = YAHOO.util.Dom.get("vw-starttime").value;

	YAHOO.util.Connect.asyncRequest("POST", fetch_ajax_url("wiki_ajax.php?do=quickedit&p=" + this.postid + "&section=" + this.sectid),
	{
		success: this.display_editor,
		failure: this.error_opening_editor,
		timeout: 15000,
		scope: this
	}, SESSIONURL
		+ 'do=quickedit&p=' + this.postid
		+ '&section=' + this.sectid
		+ '&securitytoken=' + PHP.urlencode(SECURITYTOKEN)
		+ '&editorid=' + PHP.urlencode(this.editorid)
		+ '&fromheader=' + (this.qe_obj == "forum_header" ? 1 : 0)
		+ '&poststarttime=' + poststarttime
	);
};

/**
* Handle unspecified AJAX server error when opening editor - opens full editor
*
* @param	object	YUI AJAX
*/
vB_AJAX_QuickSection.prototype.error_opening_editor = function(ajax)
{
	vBulletin_AJAX_Error_Handler(ajax);

	window.location = this.ajax_load_url[0] + this.splitter + construct_phrase(this.ajax_load_url[1], "edit") + "&section=" + this.sectid;
}

/**
* Handle unspecified AJAX server error when saving
*
* @param	object	YUI AJAX
*/
vB_AJAX_QuickSection.prototype.handle_save_error = function(ajax)
{
	vBulletin_AJAX_Error_Handler(ajax);
	
	this.show_advanced = false;
	this.full_edit();
}

/**
* Display the editor HTML when AJAX says fetch_editor() is ready
*
* @param	object	YUI AJAX
*
* @return	boolean
*/
vB_AJAX_QuickSection.prototype.display_editor = function(ajax)
{
	if (ajax.responseXML)
	{
		document.body.style.cursor = 'auto';

		var failUrl = this.ajax_load_url[0] + this.splitter + construct_phrase(this.ajax_load_url[1], "edit") + "&section=" + this.sectid;
		if (fetch_tag_count(ajax.responseXML, 'disabled'))
		{
			// this will fire if quick edit has been disabled after the showthread page is loaded
			window.location = failUrl;
		}
		else if (fetch_tag_count(ajax.responseXML, 'error'))
		{
			// do nothing
		}
		else
		{
			var editor = fetch_tags(ajax.responseXML, 'editor')[0];

			if (typeof editor == 'undefined')
			{
				window.location = failUrl;
			}

			var reason = editor.getAttribute('reason');

			// display the editor
			this.messageobj.innerHTML = editor.firstChild.nodeValue;

			// display the reason
			var reasonfield = YAHOO.util.Dom.get(this.editorid + '_edit_reason');
			if (reasonfield)
			{
				this.unchanged_reason = PHP.unhtmlspecialchars(reason);
				reasonfield.value = this.unchanged_reason;
				reasonfield.onkeypress = vB_AJAX_QuickSection_Events.prototype.reason_key_trap;
			}

			if (this.is_cke)
			{
				// ckeditor config
				if (typeof JSON == 'object' && typeof JSON.parse == 'function')
				{
					var ckeconfig = JSON.parse(fetch_tags(ajax.responseXML, 'ckeconfig')[0].firstChild.nodeValue);
				}
				else
				{
					var ckeconfig = eval('('+fetch_tags(ajax.responseXML, 'ckeconfig')[0].firstChild.nodeValue+')');
				}

				// initialize the editor
				vB_Editor[this.editorid] = new vB_Text_Editor(
					this.editorid,
					ckeconfig
				);
			}
			else
			{
				// initialize the editor
				vB_Editor[this.editorid] = new vB_Text_Editor(
					this.editorid,
					editor.getAttribute('mode'),
					editor.getAttribute('parsetype'),
					editor.getAttribute('parsesmilies')
				);
			}

			if (!this.is_cke || vB_Editor[this.editorid].editorready)
			{
				this.display_editor_final('editorready', null, this);
			}
			else
			{
				vB_Editor[this.editorid].vBevents.editorready.subscribe(this.display_editor_final, this);
			}
		}
	}

	return false;
};

vB_AJAX_QuickSection.prototype.display_editor_final = function(type, args, me)
{
	if (!VAULT_40X_COMPAT)
	{
		vB_Editor[me.editorid].set_editor_width('99%', true);
	}

	var cureditor = YAHOO.util.Dom.get(me.editorid + '_editor');
	var curmsg = YAHOO.util.Dom.get(me.editorid + '_textarea');

	if (cureditor)
	{
		if (!VAULT_40X_COMPAT && !is_ie)
		{
			YAHOO.util.Dom.addClass(cureditor, "QuickSection");
		}

		YAHOO.util.Dom.setStyle(curmsg, "width", (VAULT_40X_COMPAT ? "100%" : "99%"));

		var edithead = YAHOO.util.Dom.getElementsBy(function(e)
		{
			if (e && e.name && e.name == "vB::QuickSection::" + vB_QuickSection.postid + ":" + vB_QuickSection.sectid)
			{
				return true;
			}

			return false;
		}, "a")[0];

		if (edithead)
		{
			// link.span.hX
			edithead = edithead.parentNode.parentNode;
			if (edithead.scrollIntoView)
			{
				edithead.scrollIntoView(true);
			}

			if (
				vB_Section &&
				YAHOO.util.Dom.getStyle("collapseobj_" + vB_QuickSection.postid + "_" + vB_QuickSection.sectid, "display") == "none"
			)
			{
				vB_Section.toggle(vB_QuickSection.postid, vB_QuickSection.sectid);
			}
		}
	}

	vB_Editor[me.editorid].check_focus();
	me.unchanged = vB_Editor[me.editorid].get_editor_contents();

	YAHOO.util.Event.on(YAHOO.util.Dom.get(me.editorid + '_save'), "click", me.save, me, true);
	YAHOO.util.Event.on(YAHOO.util.Dom.get(me.editorid + '_abort'), "click", me.abort, me, true);
	YAHOO.util.Event.on(YAHOO.util.Dom.get(me.editorid + '_adv'), "click", me.full_edit, me, true);
	YAHOO.util.Event.on(YAHOO.util.Dom.get('ajax_wiki_errors_closebtn'), "click", me.hide_errors, me, true);
	YAHOO.util.Event.on(YAHOO.util.Dom.get('ajax_wiki_errors_cancel'), "click", me.abort, me, true);

	if (VAULT_40X_COMPAT)
	{
		init_popupmenus(YAHOO.util.Dom.get(me.editorid));
	}
};

/**
* Destroy the editor, and use the specified text as the post contents
*
* @param	ajax	Quick edit response
* @param	string	Type of restore
*/
vB_AJAX_QuickSection.prototype.restore = function(ajax_response, type)
{
	this.hide_errors(true);
	if (this.editorid && vB_Editor[this.editorid] && vB_Editor[this.editorid].initialized)
	{
		vB_Editor[this.editorid].destroy();
	}

	if (type == 'tableobj')
	{
		// usually called when message is saved
		var failUrl = vB_QuickSection.ajax_load_url[0] + this.splitter + vB_QuickSection.ajax_load_url[1].split("&do")[0];
		if (!ajax_response)
		{
			window.location = failUrl;
		}

		var tab_id;
		var articleResponse = fetch_tags(ajax_response, "article")[0];
		var articleId = fetch_tags(ajax_response, "url")[0];

		if (!articleResponse || !articleId)
		{
			window.location = failUrl;
		}

		var tabId = articleResponse.getAttribute("tab_id");
		if (!tabId)
		{
			window.location = failUrl;
		}

		if (tabId == "forum_header")
		{
			var collapseobj = YAHOO.util.Dom.get(tabId);
		}
		else
		{
			var collapseobj = YAHOO.util.Dom.get("collapseobj_" + tabId);
		}

		if (!collapseobj)
		{
			window.location = failUrl;
		}

		var threadurl = articleId.firstChild.nodeValue.replace(/\{(\d+)(:\w+)?\}/gi, '%$1$s');
		if (!threadurl)
		{
			window.location = failUrl;
		}

		if (tabId == "forum_header")
		{
			collapseobj.parentNode.innerHTML = articleResponse.firstChild.nodeValue;
			collapseobj = YAHOO.util.Dom.get(tabId).parentNode;
		}
		else
		{
			vB_ResetHandle.parseRedirect(ajax_response, threadurl);
			collapseobj = YAHOO.util.Dom.get("collapseobj_" + tabId);
		}

		vB_ResetHandle.parseScript(collapseobj, fetch_tags(ajax_response, "popups")[0]);
	}
	else
	{
		// usually called when message edit is cancelled
		this.messageobj.innerHTML = ajax_response;
	}

	vB_QuickSection.Section_Init(fetch_object(vB_QuickSection.qe_obj));

	this.editstate = false;
};

/**
* Cancel the post edit and restore everything to how it started
*
* @param	event	Event object
*/
vB_AJAX_QuickSection.prototype.abort = function(e)
{
	if (e)
	{
		YAHOO.util.Event.stopEvent(e);
	}

	document.body.style.cursor = 'auto';
	vB_QuickSection.restore(vB_QuickSection.originalhtml, 'messageobj');
};

/**
* Pass the edits along to the full editpost.php interface
*
* @param	event	Event object
*/
vB_AJAX_QuickSection.prototype.full_edit = function(e)
{
	var form = new vB_Hidden_Form(vB_QuickSection.ajax_load_url[0] + vB_QuickSection.splitter + construct_phrase(vB_QuickSection.ajax_load_url[1], "edit") + "&section=" + vB_QuickSection.sectid);

	form.add_variable('action', 'updatepost');
	form.add_variable('s', fetch_sessionhash());

	if (vB_QuickSection.show_advanced)
	{
		form.add_variable('advanced', 1);
	}
	else
	{
		// a quick edit, but not an ajax quick edit
		form.add_variable('quickeditnoajax', 1);
	}

	form.add_variable('section', vB_QuickSection.sectid);
	form.add_variable('securitytoken', SECURITYTOKEN);
	form.add_variable('wysiwyg', 0);
	form.add_variable('message', vB_Editor[vB_QuickSection.editorid].get_editor_contents());
	form.add_variable('minor', (fetch_object(vB_QuickSection.editorid + '_minor').checked ? 1 : 0));
	form.add_variable('reason', fetch_object(vB_QuickSection.editorid + '_edit_reason').value);
	form.add_variable('poststarttime', fetch_object(vB_QuickSection.editorid + '_starttime').value);

	form.submit_form();
};

/**
* Save the edited post via AJAX
*
* @param	event	Event object
*/
vB_AJAX_QuickSection.prototype.save = function(e)
{
	var newtext = vB_Editor[vB_QuickSection.editorid].get_editor_contents();
	var newreason = vB_Editor[vB_QuickSection.editorid];

	if (newtext == vB_QuickSection.unchanged && newreason == vB_QuickSection.unchanged_reason)
	{
		vB_QuickSection.abort(e);
	}
	else
	{
		fetch_object(vB_QuickSection.editorid + '_posting_msg').style.display = '';
		document.body.style.cursor = 'wait';

		pc_obj = fetch_object('postcount' + vB_QuickSection.postid);

		this.ajax_req = YAHOO.util.Connect.asyncRequest("POST", vB_QuickSection.ajax_load_url[0] + vB_QuickSection.splitter + construct_phrase(vB_QuickSection.ajax_load_url[1], "edit"), {
			success: vB_QuickSection.update,
			failure: vB_QuickSection.handle_save_error,
			timeout: 15000,
			scope: vB_QuickSection
		}, construct_phrase(vB_QuickSection.ajax_load_url[1], "edit")
			+ "&action=updatepost&ajax=1&wysiwyg=0"
			+ '&section=' + vB_QuickSection.sectid
			+ '&message=' + PHP.urlencode(newtext)
			+ '&securitytoken=' + PHP.urlencode(SECURITYTOKEN)
			+ '&minor=' + (fetch_object(vB_QuickSection.editorid + '_minor').checked ? 1 : 0)
			+ '&reason=' + PHP.urlencode(fetch_object(vB_QuickSection.editorid + '_edit_reason').value)
			+ '&poststarttime=' + PHP.urlencode(fetch_object(vB_QuickSection.editorid + '_starttime').value)
			+ (pc_obj != null ? '&postcount=' + PHP.urlencode(pc_obj.name) : '')
			+ '&fromheader=' + (vB_QuickSection.qe_obj == "forum_header" ? 1 : 0)
		);

		vB_QuickSection.pending = true;
	}
};

/**
* Check for errors etc. and initialize restore when AJAX says save() is complete
*
* @param	object	YUI AJAX
*
* @return	boolean	false
*/
vB_AJAX_QuickSection.prototype.update = function(ajax)
{
	if (ajax.responseXML)
	{
		vB_QuickSection.pending = false;
		document.body.style.cursor = 'auto';
		fetch_object(vB_QuickSection.editorid + '_posting_msg').style.display = 'none';

		// this is the nice error handler, of which Safari makes a mess
		if (fetch_tag_count(ajax.responseXML, 'error'))
		{
			var errors = fetch_tags(ajax.responseXML, 'error');

			var error_html = '<ol>';
			for (var i = 0; i < errors.length; i++)
			{
				error_html += '<li>' + errors[i].firstChild.nodeValue + '</li>';
			}
			error_html += '</ol>';

			vB_QuickSection.show_errors('<ol>' + error_html + '</ol>');
		}
		else
		{
			if (vB_ResetHandle)
			{
				vB_QuickSection.restore(ajax.responseXML, 'tableobj');
			}
			else
			{
				window.location = vB_QuickSection.ajax_load_url[0] + vB_QuickSection.splitter + vB_QuickSection.ajax_load_url[1].split("&do")[0];
			}
		}
	}

	return false;
};

/**
* Pop up a window showing errors
*
* @param	string	Error HTML
*/
vB_AJAX_QuickSection.prototype.show_errors = function(errortext)
{
	// add the 'unselectable' attribute to the ajax errors close button
	set_unselectable('ajax_wiki_errors_closebtn');

	YAHOO.util.Dom.get('ajax_wiki_errors_message').innerHTML = errortext;

	var errortable = YAHOO.util.Dom.get('ajax_wiki_errors');
	var measurer = (is_saf ? 'body' : 'documentElement');

	errortable.style.left = (is_ie ? document.documentElement.clientWidth : self.innerWidth) / 2 - 200 + document[measurer].scrollLeft + 'px';
	errortable.style.top = (is_ie ? document.documentElement.clientHeight : self.innerHeight) / 2 - 150 + document[measurer].scrollTop + 'px';
	errortable.style.zIndex = 500;
	errortable.style.position = 'absolute';

	if (VAULT_40X_COMPAT)
	{
		YAHOO.util.Dom.removeClass(errortable, "hidden");
	}
	else
	{
		errortable.style.width = '400px';
		errortable.style.display = '';
	}
};

/**
* Hide the error window
*/
vB_AJAX_QuickSection.prototype.hide_errors = function(skip_focus_check)
{
	this.errors = false;

	if (VAULT_40X_COMPAT)
	{
		YAHOO.util.Dom.addClass("ajax_wiki_errors", "hidden");
	}
	else
	{
		YAHOO.util.Dom.get("ajax_wiki_errors").style.display = 'none';
	}

	if (skip_focus_check != true)
	{
		vB_Editor[this.editorid].check_focus();
	}
};

// =============================================================================
// vB_AJAX_QuickSection Event Handlers

/**
* Class to handle quick editor events
*/
function vB_AJAX_QuickSection_Events()
{
}

/**
* Handles clicks on edit buttons of postbits
*/
vB_AJAX_QuickSection_Events.prototype.editbutton_click = function(e)
{
	return vB_QuickSection.edit(this.name);
};

/**
* Key trapper for reason box
*/
vB_AJAX_QuickSection_Events.prototype.reason_key_trap = function(e)
{
	e = e ? e : window.event;

	switch (e.keyCode)
	{
		case 9: // tab
		{
			YAHOO.util.Dom.get(vB_QuickSection.editorid + '_save').focus();
			return false;
		}
		break;

		case 13: // enter
		{
			vB_QuickSection.save();
			return false;
		}
		break;

		default:
		{
			return true;
		}
	}
}

/**
* This function runs all the necessary Javascript code on a PostBit
* after it has been loaded via AJAX. Don't use this method before a
* complete page load or you'll have problems.
*
* @param	object	Object containing postbits
*/
vB_AJAX_QuickSection.prototype.Section_Init = function(obj)
{
	console.log("Section Init: %d", this.postid + '-' + this.sectid);

	if (VAULT_40X_COMPAT)
	{
		YAHOO.vBulletin.vBPopupMenu.instrument(obj);
	}

	if (typeof vB_QuickSection != 'undefined')
	{
		// init quick edit controls
		vB_AJAX_QuickSection_Init(obj);
	}

	if (typeof vB_Lightbox != 'undefined')
	{
		init_postbit_lightbox(obj);
	}

	child_img_alt_2_title(obj);
};

// #############################################################################
// initialize the editor class

var vB_QuickSection = new vB_AJAX_QuickSection();
var section_holders = [];

/*======================================================================*\
 * Downloaded: 19:45, Wed Apr 11th 2012
 * Licensed to: http://www.enworld.org
\*======================================================================*/